@@ -565,117 +565,6 @@ apply_regime <- function(selection_df, regime_condition, partial_weight = 0) {
 
 
 
-
-#' Limit the number of positions in a selection matrix
-#'
-#' @description
-#' This function enforces position limits, keeping only the top N securities
-#' when more are selected.
-#'
-#' @param selection_df Binary selection matrix
-#' @param max_positions Maximum number of positions allowed
-#' @param ranking_signal DataFrame with values for ranking (if NULL, selections are random)
-#' @param verbose Print information about position limiting (default: FALSE)
-#'
-#' @return Selection matrix with at most max_positions securities selected per period
-#' @export
-#' @examples
-#' data("sample_prices_weekly")
-#' momentum <- calc_momentum(sample_prices_weekly, 12)
-#' # Create a selection of top 30 stocks
-#' my_selections <- filter_top_n(momentum, 30)
-#' # Limit to 20 positions, ranked by momentum
-#' concentrated <- limit_positions(my_selections, 20, momentum)
-#' # Limit to 10 positions, keeping existing selections randomly
-#' limited <- limit_positions(my_selections, 10)
-limit_positions <- function(selection_df, max_positions, ranking_signal = NULL, verbose = FALSE) {
-  # Input validation
-  validate_signal_data(selection_df, "limit_positions")
-
-  if (!is.numeric(max_positions) || max_positions <= 0) {
-    stop("limit_positions: max_positions must be a positive number")
-  }
-
-  if (!is.null(ranking_signal)) {
-    validate_signal_data(ranking_signal, "limit_positions")
-    if (!identical(dim(selection_df), dim(ranking_signal))) {
-      stop("limit_positions: selection_df and ranking_signal must have same dimensions")
-    }
-  }
-
-  # Ensure data.table
-  selection_dt <- ensure_dt_copy(selection_df)
-  symbol_cols <- setdiff(names(selection_dt), "Date")
-
-  if (!is.null(ranking_signal)) {
-    ranking_dt <- ensure_dt_copy(ranking_signal)
-  }
-
-  # Count periods that need limiting
-  n_selected_per_period <- rowSums(selection_dt[, ..symbol_cols])
-  periods_over_limit <- sum(n_selected_per_period > max_positions)
-
-  if (verbose && periods_over_limit > 0) {
-    cat(sprintf("limit_positions: Reducing positions in %d periods (%.1f%% of total)\n",
-                periods_over_limit,
-                100 * periods_over_limit / nrow(selection_dt)))
-  }
-
-  # Process only periods that exceed the limit
-  result_df <- copy(selection_dt)
-
-  for (i in which(n_selected_per_period > max_positions)) {
-    # Get current selections
-    period_selections <- unlist(result_df[i, ..symbol_cols])
-    selected_symbols <- symbol_cols[period_selections == 1]
-    n_current <- length(selected_symbols)
-
-    if (is.null(ranking_signal)) {
-      # No ranking signal - randomly select max_positions
-      keep_symbols <- sample(selected_symbols, max_positions)
-      drop_symbols <- setdiff(selected_symbols, keep_symbols)
-    } else {
-      # Rank by signal and keep top max_positions
-      period_signals <- unlist(ranking_dt[i, ..selected_symbols])
-
-      # Handle NAs in ranking signal
-      if (any(is.na(period_signals))) {
-        # Put NAs at the end (lowest priority)
-        na_symbols <- selected_symbols[is.na(period_signals)]
-        valid_symbols <- selected_symbols[!is.na(period_signals)]
-        valid_signals <- period_signals[!is.na(period_signals)]
-
-        # Sort valid symbols by signal
-        sorted_indices <- order(valid_signals, decreasing = TRUE)
-        sorted_symbols <- c(valid_symbols[sorted_indices], na_symbols)
-      } else {
-        # All signals valid - simple sort
-        sorted_indices <- order(period_signals, decreasing = TRUE)
-        sorted_symbols <- selected_symbols[sorted_indices]
-      }
-
-      # Keep top max_positions
-      keep_symbols <- sorted_symbols[1:max_positions]
-      drop_symbols <- sorted_symbols[(max_positions + 1):length(sorted_symbols)]
-    }
-
-    # Update selections
-    if (length(drop_symbols) > 0) {
-      result_df[i, (drop_symbols) := 0]
-    }
-
-    if (verbose) {
-      cat(sprintf("  Period %d: Reduced from %d to %d positions\n",
-                  i, n_current, max_positions))
-    }
-  }
-
-  return(result_df)
-}
-
-
-
-
 #' Filter by Percentile
 #'
 #' @description
